@@ -5,10 +5,10 @@
  *    combinations.
  *
  *  The dos2unix package is distributed under FreeBSD style license.
- *  See also http://www.freebsd.org/copyright/freebsd-license.html
+ *  See also https://www.freebsd.org/copyright/freebsd-license.html
  *  --------
  *
- *  Copyright (C) 2009-2016 Erwin Waterlander
+ *  Copyright (C) 2009-2018 Erwin Waterlander
  *  Copyright (C) 1994-1995 Benjamin Lin.
  *  All rights reserved.
  *
@@ -69,7 +69,7 @@ void PrintLicense(void)
   D2U_ANSI_FPRINTF(stdout,_("\
 Copyright (C) 2009-%d Erwin Waterlander\n\
 Copyright (C) 1994-1995 Benjamin Lin\n\
-All rights reserved.\n\n"),2016);
+All rights reserved.\n\n"),2023);
   PrintBSDLicense();
 }
 
@@ -477,11 +477,16 @@ int main (int argc, char *argv[])
   CFlag *pFlag;
   char *ptr;
   char localedir[1024];
+  int ret;
 # ifdef __MINGW64__
   int _dowildcard = -1; /* enable wildcard expansion for Win64 */
 # endif
   int  argc_new;
   char **argv_new;
+#ifdef D2U_UNIFILE
+  wchar_t **wargv;
+  char ***argv_glob;
+#endif
 
   progname[8] = '\0';
   strcpy(progname,"unix2dos");
@@ -489,14 +494,14 @@ int main (int argc, char *argv[])
 #ifdef ENABLE_NLS
    ptr = getenv("DOS2UNIX_LOCALEDIR");
    if (ptr == NULL)
-      strcpy(localedir,LOCALEDIR);
+      d2u_strncpy(localedir,LOCALEDIR,sizeof(localedir));
    else {
       if (strlen(ptr) < sizeof(localedir))
-         strcpy(localedir,ptr);
+         d2u_strncpy(localedir,ptr,sizeof(localedir));
       else {
          D2U_UTF8_FPRINTF(stderr,"%s: ",progname);
          D2U_ANSI_FPRINTF(stderr, "%s", _("error: Value of environment variable DOS2UNIX_LOCALEDIR is too long.\n"));
-         strcpy(localedir,LOCALEDIR);
+         d2u_strncpy(localedir,LOCALEDIR,sizeof(localedir));
       }
    }
 #endif
@@ -542,8 +547,6 @@ int main (int argc, char *argv[])
 
 #ifdef D2U_UNIFILE
   /* Get arguments in wide Unicode format in the Windows Command Prompt */
-  wchar_t **wargv;
-  char ***argv_glob;
 
   /* This does not support wildcard expansion (globbing) */
   wargv = CommandLineToArgvW(GetCommandLineW(), &argc);
@@ -552,6 +555,7 @@ int main (int argc, char *argv[])
   if (argv_glob == NULL) {
     D2U_UTF8_FPRINTF(stderr, "%s:", progname);
     D2U_ANSI_FPRINTF(stderr, " %s\n", strerror(errno));
+    free(pFlag);
     return errno;
   }
   /* Glob the arguments and convert them to UTF-8 */
@@ -563,9 +567,11 @@ int main (int argc, char *argv[])
 #endif
 
 #ifdef D2U_UNICODE
-  return parse_options(argc_new, argv_new, pFlag, localedir, progname, PrintLicense, ConvertUnixToDos, ConvertUnixToDosW);
+  ret = parse_options(argc_new, argv_new, pFlag, localedir, progname, PrintLicense, ConvertUnixToDos, ConvertUnixToDosW);
 #else
-  return parse_options(argc_new, argv_new, pFlag, localedir, progname, PrintLicense, ConvertUnixToDos);
+  ret = parse_options(argc_new, argv_new, pFlag, localedir, progname, PrintLicense, ConvertUnixToDos);
 #endif
+  free(pFlag);
+  return ret;
 }
 
